/* es drivers for X */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <stdlib.h>
#include "es.h"

//#define BPP	15
#define BPP	16

Display	*dis;
int	screen;
Window	win;
XEvent	event;
GC	gc;
KeySym	key;

int bp;
long lfg,lbg;

int es_hasmode()
{
 return 1;
}

char * es_getmodename()
{
 char *s;
 
 s=(char *) malloc(strlen("this shouldn't happen")+1);
 memset(s,0,sizeof(s));
 strcat(s,"this shouldn't happen");
 
 return s;
}

long es_color(int r,int g,int b)
{
 if (BPP == 16)
 return (b*31/255)*1	+
        (g*63/255)*32	+
        (r*31/255)*32*64;
 if (BPP == 15)
 return (b*31/255)*1	+
        (g*31/255)*32	+
        (r*31/255)*32*32; 
}

void es_setfontcolors(long bg, long fg)
{
 lfg=fg; lbg=bg;
 XSetForeground(dis,gc,fg);
 XSetBackground(dis,gc,bg);
}

void es_setpixel(int x, int y, long c)
{
 XSetForeground(dis,gc,c);
 XDrawPoint(dis,win,gc,x,y);
}

void es_setpixelrgb(int x, int y, int r, int g, int b)
{
 if (x>-1 && x<640 &&
     y>-1 && y<480   )
 {
  XSetForeground(dis,gc,es_color(r,g,b));
  XDrawPoint(dis,win,gc,x,y);
 }
}

void es_write(int x, int y, char *s)
{
 int tx,ty;
 XSetForeground(dis,gc,lbg);
 for (tx=x; tx<x+strlen(s)*8; tx++)
  for (ty=y; ty<y+8; ty++)
   XDrawPoint(dis,win,gc,tx,ty);
 XSetForeground(dis,gc,lfg);
 XDrawString(dis,win,gc,x+1,y+7,s,strlen(s));
}

int es_checkbutton(int status)
{
 switch(status)
 {
  case Button1:	return ES_LEFTBUTTON;
  case Button2:	return ES_MIDDLEBUTTON;
  case Button3:	return ES_RIGHTBUTTON;
  default:	return ES_NOBUTTON;
 }
}

void redraw()
{
 int x,y;
 
 while (event.type==Expose)
 {
  for (x=0; x<event.xexpose.width; x++)
   for (y=0; y<event.xexpose.height; y++)
    es_pixel(event.xexpose.x+x,event.xexpose.y+y);

  XNextEvent(dis,&event);
 } 
}


int es_getch()
{
 unsigned char c[1];
 
 event.type=0;

 while (event.type!=KeyPress)
 {
  if (event.type==Expose) redraw();
  if (event.type==ButtonRelease) bp=0;
  XNextEvent(dis,&event);
 }
 XLookupString(&event.xkey,c,1,&key,0); 

 if (c[0]==13) return 10;
 if (c[0]==8) return 127;

 return c[0];
}

Cursor createnullcursor ( Display*	display, Window	root )
{
    Pixmap cursormask;
    XGCValues xgc;
    GC gc;
    XColor dummycolour;
    Cursor cursor;

    cursormask = XCreatePixmap(display, root, 1, 1, 1/*depth*/);
    xgc.function = GXclear;
    gc =  XCreateGC(display, cursormask, GCFunction, &xgc);
    XFillRectangle(display, cursormask, gc, 0, 0, 1, 1);
    dummycolour.pixel = 0;
    dummycolour.red = 0;
    dummycolour.flags = 04;
    cursor = XCreatePixmapCursor(display, cursormask, cursormask,
				 &dummycolour,&dummycolour, 0,0);
    XFreePixmap(display,cursormask);
    XFreeGC(display,gc);
    return cursor;
}


void es_init()
{
 XGCValues *tmp;
// XSetWindowAttributes *t;
 
 dis=XOpenDisplay((char *)0);
 screen=DefaultScreen(dis);
 
 win=XCreateSimpleWindow(dis,
                         DefaultRootWindow(dis),
                         0,0,
                         640,480,
                         5,
                         es_color(0,0,0),
                         es_color(0,0,0));
 
 
 XSetStandardProperties(dis,win,"Editskin","Editskin",None,NULL,0,NULL);
 
 XSelectInput(dis,win,ExposureMask	|
                      ButtonPressMask	|
                      ButtonReleaseMask	|
                      KeyPressMask	|
                      PointerMotionMask	 );

 
 gc=XCreateGC(dis,win,0,0);
 
 tmp=(XGCValues *) malloc(1);
 
 tmp->font=XLoadFont(dis,"-misc-fixed-medium-r-normal--9-*-*-*-c-0-iso8859-*");
 
 XChangeGC(dis,gc,GCFont,tmp);
 
 XSetBackground(dis,gc,es_color(255,255,255));
 XSetForeground(dis,gc,es_color(0,0,0));

 XDefineCursor(dis,win,createnullcursor(dis,win));

 
 XClearWindow(dis,win);
 XMapRaised(dis,win);
}

void es_close()
{
 XFreeGC(dis,gc);
 XDestroyWindow(dis,win);
 XCloseDisplay(dis);
 exit(0);
}

es_event es_handler(es_event t)
{
 es_event tmp;
 unsigned char c[1];
 
 tmp=t;
 
 tmp.key=0;
 
 XNextEvent(dis,&event);

 if (event.type==Expose) // Keep this event first in the list!
  redraw();

 if (event.type==KeyPress && XLookupString(&event.xkey,c,1,&key,0)==1)
  tmp.key=c[0];
  
 if (event.type==ButtonPress)
 {
  tmp.mb=event.xbutton.button;
  bp=1;
 } else
  if (bp==0) tmp.mb=ES_NOBUTTON;

 if (event.type==ButtonRelease)
 {
  bp=0;
  tmp.mb=ES_NOBUTTON;
 }
  
 if (event.type==MotionNotify)
 {
  tmp.mx=event.xmotion.x;
  tmp.my=event.xmotion.y;
 }  

 return tmp;
}

